#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun  1 14:20:52 2021

@author: peruzzetto
"""

import matplotlib.pyplot as plt
import numpy as np

import os
import importlib

import tilupy.notations as notations
import tilupy.plot as plt_fn
import tilupy.raster

RAW_STATES = ['hvert', 'h', 'ux', 'uy']

TEMPORAL_DATA_0D = ['hu2int', 'vol']
TEMPORAL_DATA_1D = ['']
TEMPORAL_DATA_2D = ['hvert', 'h', 'u', 'ux', 'uy', 'hu', 'hu2']
STATIC_DATA_0D = []
STATIC_DATA_1D = []
STATIC_DATA_2D = []

TOPO_DATA_2D = ['z', 'zinit', 'costh']

NP_OPERATORS = ['max', 'mean', 'std', 'sum', 'min']
OTHER_OPERATORS = ['final', 'initial']

COMPUTED_STATIC_DATA_2D = []

for stat in NP_OPERATORS + OTHER_OPERATORS:
    for name in TEMPORAL_DATA_2D:
        COMPUTED_STATIC_DATA_2D.append(name + '_' + stat)
        
STATIC_DATA_2D += COMPUTED_STATIC_DATA_2D

DATA_NAMES = TEMPORAL_DATA_0D + TEMPORAL_DATA_1D + TEMPORAL_DATA_2D
DATA_NAMES += STATIC_DATA_0D + STATIC_DATA_1D + STATIC_DATA_2D

class TemporalResults:
    """ Time dependent result of simulation """

    def __init__(self, name, d, t, x=None, y=None, z=None):
        # 0d, 1d or 2d result, plus one dimension for time.
        self.d = d
        # 1d array with times, matching last dimension of self.d
        self.t = t
        # Name of data (e.g. h, u, hu, ...)
        self.name = name
        # x and y arrays
        self.x = x
        self.y = y
        # topography
        self.z = z

    def get_temporal_stat(self, stat):
        """ Statistical analysis along temporal dimension """
        if stat in NP_OPERATORS:
            dnew = getattr(np, stat)(self.d, axis=-1)
        elif stat == 'final':
            dnew = self.d[..., -1]
        elif stat == 'initial':
            dnew = self.d[..., 0]
        return StaticResults(self.name + '_' + stat, dnew)

    def spatial_integration(self, axis=(0, 1), cellsize=None):
        """ Spatial integration along one or two axes """
        if cellsize is None:
            dnew = np.sum(self.d, axis=axis)
        else:
            dnew = np.sum(self.d*cellsize, axis=axis)
        self.d = dnew
        
    def plot(self, axe=None, figsize=None, folder_out=None, fmt='png', dpi=150,
             x=None, y=None, z=None, sup_plt_fn=None, sup_plt_fn_args=None,
             **kwargs):
        """ Plot results as time dependent"""
        
        if axe is None:
            fig, axe = plt.subplots(1, 1, figsize=figsize)
        
        if self.d.ndim == 1:
            axe.plot(self.t, self.d, **kwargs)
            axe.set_xlabel('Time (s)')
            axe.set_ylabel(notations.LABELS[self.name])
            
        elif self.d.ndim == 2:
            raise NotImplementedError(('Plot of 1D data as time',
                                       ' functions not implemented yet'))
            
        elif self.d.ndim == 3:
            if x is None or y is None or z is None:
                raise TypeError('x, y or z data missing')
            plt_fn.plot_maps(x, y, z, self.d, self.t,
                             self.name, folder_out=folder_out, 
                             figsize=figsize, fmt=fmt, dpi=dpi,
                             sup_plt_fn=sup_plt_fn,
                             sup_plt_fn_args=sup_plt_fn_args,
                             **kwargs)
            axe, fig = None, None
            
        if self.d.ndim != 3 and sup_plt_fn is not None:
            if sup_plt_fn_args is None:
                sup_plt_fn_args = dict()
            sup_plt_fn(axe, **sup_plt_fn_args)
            
        return axe, fig
    
    def save(self, folder=None, file_name=None, fmt='asc', time=None,
             x=None, y=None, **kwargs):
        
        if self.d.ndim == 1:
            raise NotImplementedError()
            
        elif self.d.ndim == 2:
            raise NotImplementedError()
            
        elif self.d.ndim == 3:
            if x is None:
                x = self.x
            if y is None:
                y = self.y
            if x is None or y is None:
                raise ValueError('x et y arrays must not be None')
            
            if file_name is None:
                file_name = self.name
                
            if folder is not None:
                file_name = os.path.join(folder, file_name)
                
            if time is not None:
                if isinstance(time, str):
                    if time == 'final':
                        inds = [self.d.shape[2]-1]
                    elif time == 'initial':
                        inds = [0]
                else:
                    inds = [np.argmin(time - np.abs(np.array(self.t)-time))]
            
            for i in range(inds):
                file_out = file_name + '_{:04d}.'.format(i) + fmt
                tilupy.raster.write_raster(x, y, self.d[:, :, i], file_out,
                                      fmt=fmt, **kwargs)
                


class StaticResults:
    """ Result of simulation without time dependence"""

    def __init__(self, name, d, x=None, y=None, z=None):
        # 1d or 2d array
        self.d = d
        # Name of data
        self.name = name
        # x and y arrays
        self.x = x
        self.y = y
        # topography
        self.z = z
        
    def plot(self, axe=None, figsize=None,
             folder_out=None, suffix=None, prefix=None,
             fmt='png', dpi=150,
             x=None, y=None, z=None, 
             sup_plt_fn=None, sup_plt_fn_args=None, **kwargs):
        """Plot results as map"""
        
        if axe is None:
            fig, axe = plt.subplots(1, 1, figsize=figsize)
        else:
            fig = axe.figure
            
        if x is None:
            x = self.x
        if y is None:
            y = self.y
        if z is None:
            z = self.z
        
        if x is None or y is None or z is None:
            raise TypeError('x, y or z data missing')
            
        axe = plt_fn.plot_data_on_topo(x, y, z, self.d, axe=axe,
                                       figsize=figsize,
                                       **kwargs)
        if sup_plt_fn is not None:
            if sup_plt_fn_args is None:
                sup_plt_fn_args = dict()
            sup_plt_fn(axe, **sup_plt_fn_args)
        
        if folder_out is not None:
            file_name = self.name
            if suffix is not None:
                file_name = file_name + '_' + suffix
            if prefix is not None:
                file_name = prefix + '_' + file_name
            file_out = os.path.join(folder_out,file_name + '.' + fmt)
            axe.figure.tight_layout(pad=0.1)
            axe.figure.savefig(file_out, dpi=dpi, bbox_inches='tight',
                               pad_inches=0.05)
        
        return axe, fig
    
    def save(self, folder=None, file_name=None, fmt='txt', time=None,
             x=None, y=None, **kwargs):
        
        if x is None:
            x = self.x
        if y is None:
            y = self.y
        
        if x is None or y is None:
            raise ValueError('x et y arrays must not be None')
            
        if file_name is None:
            file_name = self.name + '.' + fmt
            
        if folder is not None:
            file_name = os.path.join(folder, file_name)
            
        tilupy.raster.write_raster(x, y, self.d, file_name,
                              fmt=fmt, **kwargs)
            
        

class Results:
    """ Results of thin-layer model simulation

    This class is the parent class for all simulation results, whatever the
    kind of input data. Methods and functions for processing results are given
    here. Reading results from code specific outputs is done in inhereited
    classes.
    """

    def __init__(self, *args, **kwargs):
        """
        Call init function used for code in inherited classes.

        It is not recommended

        Parameters
        ----------
        code : string
            Name of code from which results must be read.
        """
        self._h_max = None
        self._h = None
        self._costh = None
        self._zinit = None
        

    @property
    def zinit(self):
        """ Get initial topography """
        return self._zinit
    
    @property
    def z(self):
        """ Alias for zinit"""
        return self.zinit
    
    @property
    def h(self):
        """ Get thickness """
        if self._h is None:
            self._h = self.get_temporal_output('h').d
        return self._h
    
    @property
    def h_max(self):
        """ Get maximum thickness """
        if self._h_max is None:
            self._h_max = self.get_static_output('h', 'max').d
        return self._h_max
    
    def get_costh(self):
        """Get cos(slope) of topography"""
        [Fx, Fy] = np.gradient(self.zinit, self.y, self.x)
        costh = 1/np.sqrt(1 + Fx**2 + Fy**2)
        return costh

    @property
    def costh(self):
        """ Compute or get cos(slope) of topography """
        if self._costh is None:
            self._costh = self.get_costh()
        return self._costh

    def get_temporal_output(self, name, h_thresh=None):
        return TemporalResults(name, None, None, h_thresh=h_thresh)

    def get_static_output(self, name, **kwargs):
        return StaticResults(name, None, **kwargs)
    
    def get_output(self, name, **kwargs):
        if name in TEMPORAL_DATA_2D:
            data = self.get_temporal_output(name, **kwargs)
        elif name in STATIC_DATA_2D:
            state, stat = name.split('_')
            data = self.get_static_output(name, **kwargs)
        return data
    
    def plot(self, name, save=True, folder_out=None, dpi=150, fmt='png',
             h_thresh=None, from_file=False, display_plot=True,
             **kwargs):
        
        assert(name in DATA_NAMES)
        
        if save:
            if folder_out is None:
                folder_out = os.path.join(self.folder_output, 'plots')
            os.makedirs(folder_out, exist_ok=True)
            kwargs['folder_out'] = folder_out
            kwargs['dpi'] = dpi
            kwargs['fmt'] = fmt
            
        if not display_plot:
            backend = plt.get_backend()
            plt.switch_backend('Agg')
        
        data = self.get_output(name, from_file=from_file, h_thresh=h_thresh)
            
        if name in TEMPORAL_DATA_2D + STATIC_DATA_2D:
            if 'colorbar_kwargs' not in kwargs:
                kwargs['colorbar_kwargs'] = dict()
            if 'label' not in kwargs['colorbar_kwargs']:
                labels = notations.LABELS
                kwargs['colorbar_kwargs']['label'] = labels[name]
                
        
        if 'x' not in kwargs:
            kwargs['x'] = self.x
        if 'y' not in kwargs:
            kwargs['y'] = self.y
        if 'z' not in kwargs:
            kwargs['z'] = self.zinit
            
        axe, fig = data.plot(**kwargs)
        
        if not display_plot:
            plt.switch_backend(backend)
        
        return axe, fig
    
    def save(self, name, folder=None, file_name=None, fmt='txt',
             from_file=True, **kwargs):
        
        if folder is None:
            folder = os.path.join(self.folder_output, 'processed')
            os.makedirs(folder, exist_ok=True)
        
        if name in DATA_NAMES:
            data = self.get_output(name, from_file=from_file)
            if data.d.ndim>1:
                if 'x' not in kwargs:
                    kwargs['x'] = self.x
                if 'y' not in kwargs:
                    kwargs['y'] = self.y
                
            data.save(folder=folder, file_name=file_name, fmt=fmt,
                      **kwargs)
            
        elif name in TOPO_DATA_2D:
            if file_name is None:
                file_name = name
            file_out = os.path.join(folder, file_name)
            tilupy.raster.write_raster(self.x, self.y, getattr(self, name),
                                  file_out, fmt=fmt, **kwargs)
            
    
def get_results(code, **kwargs):
    """
    Get simulation results for a given code. This function calls the 
    appropriate module.

    Parameters
    ----------
    code : TYPE
        DESCRIPTION.
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    module = importlib.import_module('tilupy.models.'+code+'.read')
    return module.Results(**kwargs)

def use_thickness_threshold(simu, array, h_thresh):
    
    thickness = simu.get_temporal_output('h')
    array[thickness.d<h_thresh] = 0
    return array


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 21 12:10:46 2021

@author: peruzzetto
"""
import os
import numpy as np


def make_constant_slope(folder_out, theta=10, m_radius=50, m_height=50,
                        m_x=None, m_y=None,
                        xmax=800, ymax=550, cellsize=2.5):
    """Init topography and mass for simulation on constant slope."""
    # Create mesh
    x = np.linspace(0, xmax, int(np.round(xmax/cellsize)))
    y = np.linspace(0, ymax, int(np.round(ymax/cellsize)))
    xmesh, ymesh = np.meshgrid(x, y)

    # Topography slope
    slope = np.tan(np.deg2rad(theta))

    # Topography array
    z = -slope*(xmesh-xmax)

    # Create initial mass
    if m_x is None:
        m_x = 3*m_radius
    if m_y is None:
        m_y = ymax/2
    m = 1-(xmesh - m_x)**2/m_radius**2 - (ymesh - m_y)**2/m_radius**2
    m = np.maximum(m*m_height, 0)

    # Write headers for ascii files
    nx = z.shape[1]
    ny = z.shape[0]
    header_txt = 'ncols {:.0f}\nnrows {:.0f}\nxllcorner 0\nyllcorner 0\n'
    header_txt += 'cellsize {:.4f}\nnodata_value -99999'
    header_txt = header_txt.format(nx, ny, cellsize)

    for a, name in zip([z, m], ['topo', 'mass']):
        np.savetxt(os.path.join(folder_out, name+'.asc'), a,
                   header=header_txt,
                   comments='')



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 10 14:47:17 2022

@author: peruzzetto
"""

import numpy as np
import matplotlib.pyplot as plt

import shapely.geometry as geom
import shapely.ops

def CSI(pred, obs):
    
    ipred = pred>0
    iobs = obs>0
    
    TP = np.sum(ipred*iobs)
    FP = np.sum(ipred*~iobs)
    FN = np.sum(~ipred*iobs)
    
    return TP/(TP+FP+FN)

def diff_runout(x_contour, y_contour, point_ref, section=None,
                orientation='W-E'):
    
    npts = len(x_contour)
    contour = geom.LineString([(x_contour[i],y_contour[i])
                               for i in range(npts)])
    point = geom.Point(point_ref)
    if section is None:
        return point.distance(contour)
    elif isinstance(section, np.ndarray):
        section = geom.LineString(section)
        
    assert(isinstance(section, geom.LineString))
    section = revert_line(section, orientation)
    intersections = section.intersection(contour)
    if isinstance(intersections, geom.MultiPoint):
        intersections = geom.LineString(section.intersection(contour))
    intersections = np.array(intersections.coords)
    if orientation == 'W-E':
        i = np.argmax(intersections[:,0])
    if orientation == 'E-W':
        i = np.argmin(intersections[:,0])
    if orientation == 'S-N':
        i = np.argmax(intersections[:,1])
    if orientation == 'N-S':
        i = np.argmin(intersections[:,1])
    intersection = geom.Point(intersections[i,:])
    
    #######
    # plt.figure()
    # cont = np.array(contour.coords)
    # sec = np.array(section.coords)
    # inter = np.array(intersection.coords)
    # pt = np.array(point.coords)
    # plt.plot(cont[:,0], cont[:,1],
    #           sec[:,0], sec[:,1],
    #           pt[:,0], pt[:,1], 'o',
    #           inter[:,0], inter[:,1],'x')
    #######
    
    return section.project(intersection) - section.project(point)
    
def revert_line(line, orientation='W-E'):
    
    pt_init = line.coords[0]
    pt_end = line.coords[-1]
    if orientation == 'W-E':
        if pt_init[0]>pt_end[0]:
            line = shapely.ops.substring(line, 1, 0, normalized=True)
    elif orientation == 'E-W':
        if pt_init[0]<pt_end[0]:
            line = shapely.ops.substring(line, 1, 0, normalized=True)
    if orientation == 'S-N':
        if pt_init[1]>pt_end[1]:
            line = shapely.ops.substring(line, 1, 0, normalized=True)
    elif orientation == 'N-S':
        if pt_init[1]<pt_end[1]:
            line = shapely.ops.substring(line, 1, 0, normalized=True)
    
    return line

def get_contour(x,y,z,zlevels,indstep=1,maxdist=30, closed_contour=True):
    
    #Add sup_value at the border of the array, to make sure contour
    #lines are closed
    if closed_contour:
        z2 = z.copy()
        ni = z2.shape[0]
        nj = z2.shape[1]
        z2 = np.vstack([np.zeros((1, nj)), z2, np.zeros((1, nj))])
        z2 = np.hstack([np.zeros((ni+2, 1)), z2, np.zeros((ni+2, 1))])
        dxi = x[1]-x[0]
        dxf = x[-1]-x[-2]
        dyi = y[1] - y[0]
        dyf = y[-1] - y[-2]
        x2 = np.insert(np.append(x, x[-1]+dxf), 0, x[0]-dxi)
        y2 = np.insert(np.append(y, y[-1]+dyf), 0, y[0]-dyi)
    else:
        x2, y2, z2 = x, y, z
    
    backend = plt.get_backend()
    plt.switch_backend('Agg')
    plt.figure()
    ax=plt.gca()    
    cs=ax.contour(x2, y2, np.flip(z2, 0), zlevels)
    nn1=1
    v1=np.zeros((1,2))
    xcontour={}
    ycontour={}
    for indlevel in range(len(zlevels)):
        for p in cs.collections[indlevel].get_paths():
            if (p.vertices.shape[0]>nn1):
                v1=p.vertices
                nn1=p.vertices.shape[0]
        xc = [v1[::indstep,0]]
        yc = [v1[::indstep,1]]
        if maxdist is not None and not closed_contour:
            ddx=np.abs(v1[0,0]-v1[-1,0])
            ddy=np.abs(v1[0,1]-v1[-1,1])
            dd=np.sqrt(ddx**2+ddy**2)
            if (dd>maxdist) :
                xc[0]=None
                yc[0]=None
        xcontour[zlevels[indlevel]]=xc[0]
        ycontour[zlevels[indlevel]]=yc[0]
    plt.switch_backend(backend)
    return xcontour,ycontour


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun  2 17:57:31 2021

@author: peruzzetto
"""

import importlib
import os
import string
import numpy as np
import matplotlib.pyplot as plt
import mpl_toolkits.axes_grid1 as mplt

import tilupy.dem
import tilupy.plot as plt_fn
import tilupy.notations
import tilupy

from tilupy.notations import LABELS
from tilupy.read import STATIC_DATA_2D


def compare_spatial_results(results, name, stat, import_kwargs=None,
                            x=None, y=None, zinit=None,
                            nrows_ncols=None, figsize=None,
                            folder_out=None, file_prefix=None,
                            fmt='png', dpi=150,
                            cbar_location='bottom', cbar_size="5%",
                            cmap_intervals=None, cmap='inferno',
                            topo_kwargs=None,
                            title_loc='top right', title_x_offset=0.02,
                            title_y_offset=0.02):
    """
    Compare spatiale results of different simulations

    Parameters
    ----------
    results : TYPE
        DESCRIPTION.
    name : TYPE
        DESCRIPTION.
    import_kwargs : TYPE, optional
        DESCRIPTION. The default is None.
    x : TYPE, optional
        DESCRIPTION. The default is None.
    y : TYPE, optional
        DESCRIPTION. The default is None.
    zinit : TYPE, optional
        DESCRIPTION. The default is None.
    figsize : TYPE, optional
        DESCRIPTION. The default is None.
    folder_out : TYPE, optional
        DESCRIPTION. The default is None.
    cbar_location : TYPE, optional
        DESCRIPTION. The default is 'bottom'.
    cbar_size : TYPE, optional
        DESCRIPTION. The default is "5%".
    cmap_intervals : TYPE, optional
        DESCRIPTION. The default is None.
    cmap : TYPE, optional
        DESCRIPTION. The default is 'inferno'.
    topo_kwargs : TYPE, optional
        DESCRIPTION. The default is None.

    Returns
    -------
    None.

    """
    if x is None or y is None:
        results[0].set_axes()
        x = results[0].x
        y = results[0].y

    if zinit is None:
        results[0].set_zinit()
        zinit = results[0].zinit

    topo_kwargs = {} if topo_kwargs is None else topo_kwargs
    import_kwargs = {} if import_kwargs is None else import_kwargs
    nplots = len(results)
    if nrows_ncols is None:
        nrows_ncols = (1, nplots)

    fig = plt.figure(figsize=figsize)
    axes = mplt.AxesGrid(fig, 111,
                         nrows_ncols=nrows_ncols,
                         axes_pad=0.1,
                         share_all=True,
                         aspect='True',
                         label_mode='L',
                         cbar_location=cbar_location,
                         cbar_mode="single",
                         cbar_pad="12%",
                         cbar_size=cbar_size)

    # Titles
    alignments = title_loc.split(' ')
    ha = alignments[1]
    va = alignments[0]
    if va == 'top':
        xt = 1-title_x_offset
    elif va == 'center':
        xt = 0.5
    elif va == 'bottom':
        xt = title_x_offset
    if ha == 'right':
        yt = 1-title_y_offset
    elif ha == 'center':
        yt = 0.5
    elif ha == 'bottom':
        yt = title_y_offset

    for i, result in enumerate(results):
        data = result.get_static_output(name, stat, **import_kwargs)
        plt_fn.plot_data_on_topo(x, y, zinit, data.d,
                                 axe=axes[i],
                                 plot_colorbar=False,
                                 colorbar_kwargs={'extend': 'max'},
                                 cmap_intervals=cmap_intervals,
                                 cmap=cmap,
                                 topo_kwargs=topo_kwargs)
        letter = '({:s})'.format(string.ascii_lowercase[i])
        axes[i].text(xt, yt, '{:s} {:s}'.format(letter, result.code),
                     ha=ha, va=va,
                     fontsize=11, zorder=100,
                     bbox=dict(boxstyle="round",
                               fc="w", ec="k", pad=0.2),
                     transform=axes[i].transAxes)
        axes[i].grid(False)

    # Colorbar
    axe_cc = axes.cbar_axes[0]
    if cbar_location in ['right', 'left']:
        orientation = 'vertical'
    else:
        orientation = 'horizontal'
    fig.colorbar(axes[0].images[-1],
                 cax=axe_cc,
                 ax=axes[0],
                 orientation=orientation,
                 extend='max')
    if orientation == 'vertical':
        axe_cc.set_ylabel(LABELS[name+stat])
    else:
        axe_cc.set_xlabel(LABELS[name+stat])

    fig.tight_layout()

    if folder_out is not None:
        if file_prefix is None:
            file = name + stat + '.' + fmt
        else:
            file = file_prefix + '_' + name + stat + '.' + fmt
        fig.savefig(os.path.join(folder_out, file),
                    dpi=dpi)


def compare_simus(codes, topo_name, law,
                  rheol_params, output_name, stat_name,
                  subfolder_topo='',
                  folder_benchmark=None, folder_out=None, **kwargs):
    """Compare simulations with different codes."""

    modules = dict()
    for c in codes:
        modules[c] = importlib.import_module('tilupy.models.'+c+'.read')

    if folder_benchmark is None:
        folder_benchmark = os.getcwd()

    folder_topo = os.path.join(folder_benchmark, topo_name)

    txt_params = tilupy.notations.make_rheol_string(rheol_params, law)

    if folder_out is None:
        folder_out = os.path.join(folder_topo, subfolder_topo,
                                  'comparison', law)
        os.makedirs(folder_out, exist_ok=True)

    file = np.os.path.join(folder_topo, 'topo.asc')
    x, y, zinit, dx = tilupy.dem.read_ascii(file)

    simus = []
    for code in codes:
        folder_simus = os.path.join(folder_topo, subfolder_topo, code, law)
        simus.append(modules[code].Results(txt_params, folder_simus))

    if output_name+stat_name in STATIC_DATA_2D:
        compare_spatial_results(simus, output_name, stat_name,
                                x=x, y=y, zinit=zinit,
                                folder_out=folder_out, file_prefix=txt_params,
                                **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue May 25 15:18:31 2021

@author: peruzzetto
"""

import os
import numpy as np

import tilupy.notations
import tilupy.raster

README_PARAM_MATCH = dict(tmax='tmax',
                          CFL='cflhyp',
                          h_min='eps0',
                          dt_im_output='dt_im')

SHALTOP_LAW_ID = dict(coulomb=1,
                      voellmy=8,
                      bingham=6,
                      muI=7)


def write_params_file(params, directory=None,
                      file_name='params.txt'):
    """
    Write params file for shaltop simulations

    Parameters
    ----------
    params : TYPE
        DESCRIPTION.
    sup_data : TYPE, optional
        DESCRIPTION. The default is {}.
    directory : TYPE, optional
        DESCRIPTION. The default is None.
    file_name : TYPE, optional
        DESCRIPTION. The default is 'params.txt'.

    Returns
    -------
    None.

    """

    if directory is None:
        directory = os.getcwd()
    with open(os.path.join(directory, file_name), 'w') as file_params:
        for name in params:
            val = params[name]
            if type(val) == int or type(val) == np.int64:
                file_params.write('{:s} {:d}\n'.format(name, val))
            if type(val) == float or type(val) == np.float64:
                file_params.write('{:s} {:.8G}\n'.format(name, val))
            if type(val) == str:
                file_params.write('{:s} {:s}\n'.format(name, val))

def raster_to_shaltop_txtfile(file_in, file_out, folder_out=None):
    
    if folder_out is not None:
        file_out = os.path.join(folder_out, file_out)
        
    x, y, rast = tilupy.raster.read_raster(file_in)
    np.savetxt(file_out,
               np.reshape(np.flip(rast, axis=0), (rast.size, 1)),
               fmt='%.12G')
    
    res = dict(x0=x[0], y0=y[0], dx=x[1]-x[0], dy=y[1]-y[0],
               nx=len(x), ny=len(y))
    
    return res


def make_simus(law, rheol_params, folder_data, folder_out, readme_file):
    """
    Write shaltop initial file for simple slope test case

    Parameters
    ----------
    deltas : TYPE
        DESCRIPTION.
    folder_in : TYPE
        DESCRIPTION.
    folder_out : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    # Get topography and initial mass, and write them in Shaltop format
    zfile = os.path.join(folder_data, 'topo.asc')
    mfile = os.path.join(folder_data, 'mass.asc')
    x, y, z, dx = tilupy.raster.read_ascii(zfile)
    _, _, m, _ = tilupy.raster.read_ascii(mfile)
    np.savetxt(os.path.join(folder_out, 'z.d'), z.T.flatten())
    np.savetxt(os.path.join(folder_out, 'm.d'), m.T.flatten())

    # Get simulation parameters from README.txt and raster .asc files
    params = tilupy.notations.readme_to_params(readme_file, README_PARAM_MATCH)
    params['nx'] = len(x)
    params['ny'] = len(y)
    params['per'] = dx*len(x)
    params['pery'] = dx*len(y)
    params['file_m_init'] = '../m.d'
    params['file_z_init'] = '../z.d'

    # Folder for rheological law, and set params accordingly
    folder_law = os.path.join(folder_out, law)
    params['icomp'] = SHALTOP_LAW_ID[law]

    param_names = [param for param in rheol_params]

    texts = tilupy.notations.make_rheol_string(rheol_params, law)

    # Run shaltop file
    run_shaltop_file = os.path.join(folder_law, 'run_shaltop.sh')
    file_txt = ""

    for i in range(len(rheol_params[param_names[0]])):

        simu_text = texts[i]
        for param_name in param_names:
            params[param_name] = rheol_params[param_name][i]
        params['folder_output'] = simu_text
        folder_results = os.path.join(folder_law, simu_text)
        os.makedirs(folder_results, exist_ok=True)
        with open(os.path.join(folder_results, '.gitignore'), 'w') as fid:
            fid.write('# Ignore everything in this directory')
            fid.write('*')
            fid.write('# Except this file')
            fid.write('!.gitignore')

        write_params_file(params, directory=folder_law,
                          file_name=simu_text+'.txt')
        file_txt += 'start_time=`date +%s`\n'
        file_txt += 'shaltop "" ' + simu_text + '.txt\n'
        file_txt += 'end_time=`date +%s`\n'
        file_txt += 'elapsed_time=$(($end_time - $start_time))\n'
        file_txt += ('string_time="${start_time} ' +
                     simu_text + ' ${elapsed_time}"\n')
        file_txt += 'echo ${string_time} >> simulation_duration.txt\n\n'

    with open(run_shaltop_file, "w") as fid:
        fid.write(file_txt)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun  2 16:16:39 2021

@author: peruzzetto
"""

import matplotlib
import copy
import os

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import seaborn as sns

from mpl_toolkits.axes_grid1 import make_axes_locatable


def centered_map(cmap, vmin, vmax, ncolors=256):
    """
    Create centered colormap

    Parameters
    ----------
    cmap : TYPE
        DESCRIPTION.
    vmin : TYPE
        DESCRIPTION.
    vmax : TYPE
        DESCRIPTION.
    ncolors : TYPE, optional
        DESCRIPTION. The default is 256.

    Returns
    -------
    new_map : TYPE
        DESCRIPTION.

    """
    p = vmax/(vmax-vmin)
    npos = int(ncolors*p)
    method = getattr(plt.cm, cmap)

    colors1 = method(np.linspace(0., 1, npos*2))
    colors2 = method(np.linspace(0., 1, (ncolors-npos)*2))
    colors = np.concatenate(
        (colors2[:ncolors-npos, :], colors1[npos:, :]), axis=0)
    # colors[ncolors-npos-1,:]=np.ones((1,4))
    # colors[ncolors-npos,:]=np.ones((1,4))
    new_map = mcolors.LinearSegmentedColormap.from_list(
        'my_colormap', colors)

    return new_map


def plot_topo(z, x, y, contour_step=None, nlevels=25, level_min=None,
              step_contour_bold=0, contour_labels_properties=None,
              label_contour=True, contour_label_effect=None,
              axe=None,
              vert_exag=1, fraction=1, ndv=0, uniform_grey=None,
              contours_prop=None, contours_bold_prop=None,
              figsize=None,
              interpolation=None,
              sea_level=0, sea_color=None, alpha=1, azdeg=315, altdeg=45,
              zmin=None, zmax=None):
    """
    Plot topography with hillshading.

    Parameters
    ----------
    z : TYPE
        DESCRIPTION.
    x : TYPE
        DESCRIPTION.
    y : TYPE
        DESCRIPTION.
    contour_step : TYPE, optional
        DESCRIPTION. The default is None.
    nlevels : TYPE, optional
        DESCRIPTION. The default is 25.
    contour_labels_properties : TYPE, optional
        DESCRIPTION. The default is None.
    axe : TYPE, optional
        DESCRIPTION. The default is None.
    vert_exag : TYPE, optional
        DESCRIPTION. The default is 1.
    contour_color : TYPE, optional
        DESCRIPTION. The default is 'k'.
    contour_alpha : TYPE, optional
        DESCRIPTION. The default is 1.
    figsize : TYPE, optional
        DESCRIPTION. The default is (10,10).
    interpolation : TYPE, optional
        DESCRIPTION. The default is None.
    sea_level : TYPE, optional
        DESCRIPTION. The default is 0.
    sea_color : TYPE, optional
        DESCRIPTION. The default is None.
    alpha : TYPE, optional
        DESCRIPTION. The default is 1.
    azdeg : TYPE, optional
        DESCRIPTION. The default is 315.
    altdeg : TYPE, optional
        DESCRIPTION. The default is 45.
    plot_terrain : TYPE, optional
        DESCRIPTION. The default is False.
    cmap_terrain : TYPE, optional
        DESCRIPTION. The default is 'gist_earth'.
    fraction : TYPE, optional
        DESCRIPTION. The default is 1.
    ndv : TYPE, optional
        DESCRIPTION. The default is 0.

    Returns
    -------
    None.

    """
    dx = x[1] - x[0]
    dy = y[1] - y[0]
    im_extent = [x[0]-dx/2,
                 x[-1]+dx/2,
                 y[0]-dy/2,
                 y[-1]+dy/2]
    ls = mcolors.LightSource(azdeg=azdeg, altdeg=altdeg)

    if level_min is None:
        if contour_step is not None:
            level_min = np.ceil(np.nanmin(z)/contour_step)*contour_step
        else:
            level_min = np.nanmin(z)
    if contour_step is not None:
        levels = np.arange(level_min, np.nanmax(z), contour_step)
    else:
        levels = np.linspace(level_min, np.nanmax(z), nlevels)

    if axe is None:
        fig = plt.figure(figsize=figsize)
        axe = fig.gca()
    else:
        fig = axe.figure
    axe.set_ylabel('Y (m)')
    axe.set_xlabel('X (m)')
    axe.set_aspect('equal')

    if uniform_grey is None:
        shaded_topo = ls.hillshade(z,
                                   vert_exag=vert_exag, dx=dx, dy=dy,
                                   fraction=1)
    else:
        shaded_topo = np.ones(z.shape)*uniform_grey
    shaded_topo[z == ndv] = np.nan
    axe.imshow(shaded_topo, cmap='gray', extent=im_extent,
               interpolation=interpolation, alpha=alpha, vmin=0, vmax=1)

    if contours_prop is None:
        contours_prop = dict(alpha=0.5, colors='k',
                             linewidths=0.5)
    axe.contour(x, y, np.flip(z, axis=0), extent=im_extent,
                levels=levels,
                **contours_prop)

    if contours_bold_prop is None:
        contours_bold_prop = dict(alpha=0.8, colors='k',
                                  linewidths=0.8)

    if step_contour_bold > 0:
        lmin = np.ceil(np.nanmin(z)/step_contour_bold)*step_contour_bold
        levels = np.arange(lmin, np.nanmax(z), step_contour_bold)
        cs = axe.contour(x, y, np.flip(z, axis=0),
                         extent=im_extent,
                         levels=levels,
                         **contours_bold_prop)
        if label_contour:
            if contour_labels_properties is None:
                contour_labels_properties = {}
            clbls = axe.clabel(cs, **contour_labels_properties)
            if contour_label_effect is not None:
                plt.setp(clbls, path_effects=contour_label_effect)

    if sea_color is not None:
        cmap_sea = mcolors.ListedColormap([sea_color])
        cmap_sea.set_under(color='w', alpha=0)
        mask_sea = (z <= sea_level)*1
        if mask_sea.any():
            axe.imshow(mask_sea, extent=im_extent, cmap=cmap_sea,
                       vmin=0.5, origin='lower', interpolation='none')


def plot_data_on_topo(x, y, z, data, axe=None, figsize=(15/2.54, 15/2.54),
                      cmap=None,
                      minval=None, maxval=None, vmin=None, vmax=None,
                      minval_abs=None,
                      cmap_intervals=None, extend_cc='max',
                      topo_kwargs=None, sup_plot=None, alpha=1,
                      plot_colorbar=True, axecc=None, colorbar_kwargs=None,
                      mask=None, alpha_mask=None, color_mask='k',
                      xlims=None, ylims=None):
    """
    Plot array data on topo.

    Returns
    -------
    None.

    """
    f = copy.copy(data)

    # vmin and vmax are similar to minval and maxval
    # and supplent minval and maxval if used
    if vmin is not None:
        minval = vmin

    if vmax is not None:
        maxval = vmax

    # Remove values below and above minval and maxval, depending on whether
    # cmap_intervals are given with or without extend_cc
    if cmap_intervals is not None:
        norm = matplotlib.colors.BoundaryNorm(cmap_intervals, 256,
                                              extend=extend_cc)
        if extend_cc in ['neither', 'max']:
            minval = cmap_intervals[0]
            f[f < minval] = np.nan
        elif extend_cc in ['neither', 'min']:
            maxval = cmap_intervals[-1]
            f[f > maxval] = np.nan
    else:
        norm = None
        # if maxval is not None:
        #     f[f > maxval] = np.nan
        if minval is not None:
            f[f < minval] = np.nan

    # Get min and max values
    if maxval is None:
        maxval = np.nanmax(f)
    if minval is None:
        minval = np.nanmin(f)

    if minval_abs:
        f[np.abs(f) <= minval_abs] = np.nan
    else:
        f[f == 0] = np.nan

    # Define colormap type
    if cmap is None:
        if maxval*minval >= 0:
            cmap = 'hot_r'
        else:
            cmap = 'seismic'
    if maxval*minval >= 0:
        color_map = matplotlib.cm.get_cmap(cmap).copy()
    else:
        color_map = centered_map(cmap, minval, maxval)

    if cmap_intervals is not None:
        norm = matplotlib.colors.BoundaryNorm(cmap_intervals, 256,
                                              extend=extend_cc)
        maxval = None
        minval = None
    else:
        norm = None
    # color_map.set_under([1, 1, 1], alpha=0)

    # Initialize figure properties
    dx = x[1]-x[0]
    dy = y[1]-y[0]
    im_extent = [x[0]-dx/2, x[-1]+dx/2, y[0]-dy/2, y[-1]+dy/2]
    if axe is None:
        fig = plt.figure(figsize=figsize)
        axe = fig.gca()
    else:
        fig = axe.figure
    axe.set_ylabel('Y (m)')
    axe.set_xlabel('X (m)')
    axe.set_aspect('equal', adjustable='box')

    # Plot topo
    topo_kwargs = {} if topo_kwargs is None else topo_kwargs

    plot_topo(z, x, y, axe=axe, **topo_kwargs)

    # Plot mask
    if mask is not None:
        cmap_mask = mcolors.ListedColormap([color_mask])
        cmap_mask.set_under(color='w', alpha=0)
        axe.imshow(mask.transpose(), extent=im_extent, cmap=cmap_mask,
                   vmin=0.5, origin='lower', interpolation='none',
                   zorder=3, alpha=alpha_mask)

    # Plot data
    fim = axe.imshow(f, extent=im_extent, cmap=color_map,
                     vmin=minval, vmax=maxval, alpha=alpha,
                     interpolation='none', norm=norm, zorder=4)

    # Plot colorbar
    if plot_colorbar:
        colorbar_kwargs = {} if colorbar_kwargs is None else colorbar_kwargs
        if cmap_intervals is not None and extend_cc is not None:
            colorbar_kwargs['extend'] = extend_cc
        colorbar(fim, cax=axecc, **colorbar_kwargs)

    # Adjust axes limits
    if xlims is not None:
        axe.set_xlim(xlims[0], xlims[1])

    if ylims is not None:
        axe.set_ylim(ylims[0], ylims[1])

    return axe


def plot_maps(x, y, z, data, t, file_name, folder_out=None,
              figsize=None, dpi=None, fmt='png',
              sup_plt_fn=None,
              sup_plt_fn_args=None,
              **kwargs):
    """
    Plot and save maps of simulations outputs at successive time steps

    Parameters
    ----------
    x : TYPE
        DESCRIPTION.
    y : TYPE
        DESCRIPTION.
    data : TYPE
        DESCRIPTION.
    t : TYPE
        DESCRIPTION.
    file_name : TYPE
        DESCRIPTION.
    folder_out : TYPE
        DESCRIPTION.
    dpi : TYPE, optional
        DESCRIPTION. The default is None.
    fmt : TYPE, optional
        DESCRIPTION. The default is 'png'.

    Returns
    -------
    None.

    """

    nfigs = len(t)
    if nfigs != data.shape[2]:
        raise ValueError(
            'length of t must be similar to the last dimension of data')
    if folder_out is not None:
        file_path = os.path.join(folder_out, file_name + '_{:04d}.' + fmt)
    title_fmt = 't = {:.2f} s'

    for i in range(nfigs):
        axe = plot_data_on_topo(x, y, z, data[:, :, i], axe=None,
                                figsize=figsize,
                                **kwargs)
        axe.set_title(title_fmt.format(t[i]))
        if sup_plt_fn is not None:
            if sup_plt_fn_args is None:
                sup_plt_fn_args = dict()
            sup_plt_fn(axe, **sup_plt_fn_args)
        axe.figure.tight_layout(pad=0.1)
        if folder_out is not None:
            axe.figure.savefig(file_path.format(i), dpi=dpi,
                               bbox_inches='tight', pad_inches=0.05)


def colorbar(mappable, ax=None,
             cax=None, size="5%", pad=0.1, position='right',
             **kwargs):
    """
    Create nice colorbar matching height/width of axe.

    Parameters
    ----------
    mappable : TYPE
        DESCRIPTION.
    cax : TYPE, optional
        DESCRIPTION. The default is None.
    size : TYPE, optional
        DESCRIPTION. The default is "5%".
    pad : TYPE, optional
        DESCRIPTION. The default is 0.05.
    position : TYPE, optional
        DESCRIPTION. The default is 'right'.
    extend : TYPE, optional
        DESCRIPTION. The default is 'neither'.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    if ax is None:
        ax = mappable.axes
    fig = ax.figure
    if position in ['left', 'right']:
        orientation = 'vertical'
    else:
        orientation = 'horizontal'
    if cax is None:
        # divider = ax.get_axes_locator()
        # if divider is None:
        divider = make_axes_locatable(ax)
        cax = divider.append_axes(position, size=size, pad=pad)

    cc = fig.colorbar(mappable, cax=cax,
                      orientation=orientation, **kwargs)

    if position == 'top':
        cax.xaxis.tick_top()
        cax.xaxis.set_label_position('top')
    if position == 'left':
        cax.yaxis.tick_left()
        cax.xaxis.set_label_position('left')
    return cc


def plot_heatmaps(df, values, index, columns, aggfunc='mean',
                  figsize=None, ncols=3,
                  heatmap_kws=None, notations=None, best_values=None,
                  plot_best_value='point', text_kwargs=None):

    nplots = len(values)
    ncols = min(nplots, ncols)
    nrows = int(np.ceil(nplots/ncols))
    fig = plt.figure(figsize=figsize)
    axes = []

    for i in range(nplots):
        axe = fig.add_subplot(nrows, ncols, i+1)
        axes.append(axe)
        data = df.pivot_table(index=index, columns=columns,
                              values=values[i],
                              aggfunc=aggfunc).astype(float)
        if heatmap_kws is None:
            kws = dict()
        elif isinstance(heatmap_kws, dict):
            if values[i] in heatmap_kws:
                kws = heatmap_kws[values[i]]
            else:
                kws = heatmap_kws

        if 'cmap' not in kws:
            minval = data.min().min()
            maxval = data.max().max()
            if minval*maxval < 0:
                val = max(np.abs(minval), maxval)
                kws['cmap'] = 'seismic'
                kws['vmin'] = -val
                kws['vmax'] = val

        if 'cbar_kws' not in kws:
            kws['cbar_kws'] = dict(pad=0.03)

        if notations is None:
            kws['cbar_kws']['label'] = values[i]
        else:
            if values[i] in notations:
                kws['cbar_kws']['label'] = notations[values[i]]
            else:
                kws['cbar_kws']['label'] = values[i]

        sns.heatmap(data, ax=axe, **kws)

        if best_values is not None:
            best_value = best_values[values[i]]
            array = np.array(data)
            irow = np.arange(data.shape[0])

            if best_value == 'min':
                ind = np.nanargmin(array, axis=1)
                i2 = np.nanargmin(array[irow, ind])
            if best_value == 'min_abs':
                ind = np.nanargmin(np.abs(array), axis=1)
                i2 = np.nanargmin(np.abs(array[irow, ind]))
            elif best_value == 'max':
                ind = np.nanargmax(array, axis=1)
                i2 = np.nanargmax(array[irow, ind])

            if plot_best_value == 'point':
                axe.plot(ind + 0.5, irow+0.5, ls='',
                         marker='o', mfc='w', mec='k', mew=0.5, ms=5)
                axe.plot(ind[i2] + 0.5, i2+0.5, ls='',
                         marker='o', mfc='w', mec='k', mew=0.8, ms=9)
            elif plot_best_value == 'text':
                indx = list(ind)
                indx.pop(i2)
                indy = list(irow)
                indy.pop(i2)
                default_kwargs = dict(ha='center', va='center',
                                      fontsize=8)
                if text_kwargs is None:
                    text_kwargs = default_kwargs
                else:
                    text_kwargs = dict(default_kwargs, **text_kwargs)
                for i, j in zip(indx, indy):
                    axe.text(i + 0.5, j+0.5,
                             '{:.2g}'.format(array[j, i]),
                             **text_kwargs)
                text_kwargs2 = dict(text_kwargs, fontweight='bold')
                axe.text(ind[i2] + 0.5, i2+0.5,
                         '{:.2g}'.format(array[i2, ind[i2]]),
                         **text_kwargs2)

    axes = np.array(axes).reshape((nrows, ncols))
    for i in range(nrows):
        for j in range(1, ncols):
            axes[i, j].set_ylabel('')
            # axes[i, j].set_yticklabels([])

    for i in range(nrows-1):
        for j in range(ncols):
            axes[i, j].set_xlabel('')
            # axes[i, j].set_xticklabels([])

    if notations is not None:
        for i in range(nrows):
            axes[i, 0].set_ylabel(notations[index])
        for j in range(ncols):
            axes[-1, j].set_xlabel(notations[columns])

    fig.tight_layout()

    return fig



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 26 11:03:56 2021

@author: peruzzetto
"""

import os
import numpy as np

import tilupy.notations

readme_param_match = dict(tmax='tmax',
                          CFL='cflhyp',
                          h_min='eps0',
                          dt_im_output='dt_im')


def readme_to_params(folder_data):
    """
    Transform readme to parameters dictionnary.

    Parameters
    ----------
    folder_data : TYPE
        DESCRIPTION.

    Returns
    -------
    params : TYPE
        DESCRIPTION.

    """
    params = dict()
    with open(os.path.join(folder_data, 'README.txt'), 'r') as f:
        for line in f:
            (key, val) = line.split()
            params[key] = val
    return params


def write_params(params):
    """
    Write parameters to string for ravaflow input.

    Parameters
    ----------
    params : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    txt = ''
    for key in params:
        txt += '{:s}={:s} '.format(key, params[key])
    return txt[:-1]


def make_simus(law, rheol_params, folder_files, folder_out, readme_file):
    """
    Write shaltop initial file for simple slope test case.

    Parameters
    ----------
    deltas : TYPE
        DESCRIPTION.
    folder_in : TYPE
        DESCRIPTION.
    folder_out : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    # Parameters from README.txt file
    params_readme = tilupy.notations.readme_to_params(readme_file)
    # Parameters for simulation
    params = dict()
    # Directory where run script will be created
    folder_law = os.path.join(folder_out, law)
    os.makedirs(folder_law, exist_ok=True)

    # Prepare simulation parameters

    # Topography and initial mass
    params['elevation'] = 'elev'
    params['hrelease'] = 'minit'

    # Control parameters
    params["controls"] = '0,0,0,0,0,0'

    # Thresholds parameters
    # (min thicknees for output, min flow knetic energy for output,
    # min flow pressure for output, min thickness in simulation)
    h0 = np.float(params_readme['h_min'])
    params["thresholds"] = "{:.2E},{:.2E},{:.2E},{:.2E}".format(h0, h0, h0, h0)

    # CFL parameters
    # (CFL condition, initial time step when CFL not applicable)
    params['cfl'] = "{:.2E},0.0001".format(np.float(params_readme['CFL']))

    # Output time parameters
    params['time'] = "{:.3f},{:.2f}".format(np.float(params_readme['dt_im_output']),
                                            np.float(params_readme['tmax']))

    # Simulation phases
    params['phases'] = 's'

    # Write bash run file
    file_txt = ""
    file_path = os.path.join(folder_law, 'run.avaflow.sh')

    # Set region
    grid_params = dict()
    with open(os.path.join(folder_files, 'topo.asc'), 'r') as fid:
        for i in range(6):
            tmp = fid.readline().split()
            grid_params[tmp[0]] = float(tmp[1])
    e = grid_params['cellsize']*grid_params['ncols']
    n = grid_params['cellsize']*grid_params['nrows']
    tmp = 'g.region s=0 w=0 e={:.2f} n={:.2f} res={:.4f}\n'
    file_txt += tmp.format(e, n, grid_params['cellsize'])
    file_txt += 'g.region -s\n\n'

    # Read ascii files for mass and topography
    # Get relative path to topo and mass file, from the fodler
    # where simulations will be run. Find position of folder_files in
    # folder_out path. Simulations are run in folder_out/law/
    path = os.path.normpath(folder_out)
    ffs = path.split(os.sep)
    _, folder_topo = os.path.split(folder_files)
    n_up = len(ffs) - ffs.index(folder_topo)
    path_up = '../'*n_up
    zfile = path_up + 'topo.asc'
    mfile = path_up + 'mass.asc'
    file_txt += 'r.in.gdal --overwrite input={:s} output=elev\n'.format(zfile)
    file_txt += 'r.in.gdal --overwrite input={:s} output=minit\n'.format(mfile)

    # prefix for output folder
    prefixs = tilupy.notations.make_rheol_string(rheol_params, law)

    param_names = [param for param in rheol_params]

    txt_friction = '{:.2f},{:.2f},{:.2f}'

    for i in range(len(rheol_params[param_names[0]])):

        simu_prefix = prefixs[i]
        if law == 'coulomb':
            delta1 = rheol_params['delta1'][i]
            params['friction'] = txt_friction.format(delta1, delta1, 0)

        if law == 'voellmy':
            delta1 = rheol_params['delta1'][i]
            ksi = rheol_params['ksi'][i]
            params['friction'] = txt_friction.format(delta1, delta1,
                                                     np.log10(ksi))

        params['prefix'] = simu_prefix

        file_txt += '\n'
        file_txt += 'start_time=`date +%s`\n'
        file_txt += 'r.avaflow -e ' + write_params(params) + '\n'
        file_txt += 'end_time=`date +%s`\n'
        file_txt += 'elapsed_time=$(($end_time - $start_time))\n'
        file_txt += ('string_time="${start_time} ' +
                     simu_prefix + ' ${elapsed_time}"\n')
        file_txt += 'echo ${string_time} >> simulation_duration.txt\n\n'

    with open(file_path, "w") as fid:
        fid.write(file_txt)



# -*- coding: utf-8 -*-
"""
Created on Wed Jul 19 15:39:38 2023

@author: peruzzetto
"""

import os

import tilupy.models.shaltop.initsimus as shinit


def test_shaltop_raster_to_input(folder_data):
    
    raster_topo = os.path.join(folder_data, 'frankslide', 'rasters', 
                               'Frankslide_topography.asc')
    
    folder_simus = os.path.join(folder_data, 'frankslide', 'shaltop')
    
    file_topo_sh = os.path.join(folder_simus, 'topography.d')
    
    if os.path.isfile(file_topo_sh):
        os.remove(file_topo_sh)
    
    shinit.raster_to_shaltop_txtfile(raster_topo, file_topo_sh)
    
    assert os.path.isfile(file_topo_sh)    

def test_shaltop_make_read_param_file(folder_data):
    
    params = dict(nx=201, ny=201,
                  per=201*20,
                  pery=201*20,
                  tmax=100, # Simulation maximum time in seconds (not comutation time)
                  dt_im=10, # Time interval (s) between snapshots recordings
                  file_z_init = 'topography.d', # Name of topography input file
                  file_m_init = 'init_mass.d',# name of init mass input file
                  initz=0, # Topography is read from file
                  ipr=0, # Initial mass is read from file
                  hinit_vert=1, # Initial is given as vertical thicknesses and 
                  # must be converted to thicknesses normal to topography
                  eps0=1e-13, #Minimum value for thicknesses and velocities
                  icomp=1, # choice of rheology (Coulomb with constant basal friction)
                  x0=1000, # Min x value (used for plots after simulation is over)
                  y0=2000) # Min y value (used for plots after simulation is over)
    
    deltas = [15, 20, 25]
    folder_simus = os.path.join(folder_data, 'frankslide', 'shaltop')
    files_created = True
    
    for delta in deltas:
        params_txt = 'delta_{:05.2f}'.format(delta).replace('.', 'p')
        param_file_path = os.path.join(folder_simus,params_txt + '.txt')
        if os.path.isfile(param_file_path):
            os.remove(param_file_path)
        params['folder_output'] = params_txt # Specify folder where outputs are stored
        params['delta1'] = delta # Specify the friction coefficient
        #Write parameter file
        shinit.write_params_file(params, directory=folder_simus,
                                 file_name=params_txt + '.txt')
        
        files_created = files_created & os.path.isfile(param_file_path)
        if not files_created:
            break
        
        assert files_created



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun  3 12:20:34 2021

@author: peruzzetto
"""
import tilupy
LABELS = dict(h='Thickness (m)',
              h_max='Maximum thickness (m)',
              u_max='Maximum velocity (m)',
              h_final='Final thickness (m)',
              h_initial='Initial thickness (m)',
              u='Velocity (m s$^{-1}$)')


def get_labels(language=None):
    # LANGUAGE = language
    if language is None:
        language = tilupy.config['language']
    if language == 'english':
        labels = dict(h='Thickness (m)',
                      h_max='Maximum thickness (m)',
                      u_max='Maximum velocity (m s$^{-1}$)',
                      h_final='Final thickness (m)',
                      h_initial='Initial thickness (m)',
                      u='Velocity (m s$^{-1}$)')
    elif language == 'french':
        labels = dict(h='Epaisseur (m)',
                      h_max='Epaisseur maximale (m)',
                      u_max='Vitesse maximale (m s$^{-1}$)',
                      h_final='Epaisseur finale (m)',
                      h_initial='Epaisseur initiale (m)',
                      u='Vitesse (m s$^{-1}$)')
    return labels 

def set_labels(language=None):
    global LABELS
    LABELS = get_labels(language=language)

def readme_to_params(file, readme_param_match=None):
    """
    Convert README file as dictionnary of parameters, following matching names.

    Parameters
    ----------
    file : TYPE
        DESCRIPTION.
    readme_param_match : TYPE
        DESCRIPTION.

    Returns
    -------
    params : TYPE
        DESCRIPTION.

    """
    params = dict()
    with open(file, 'r') as f:
        if readme_param_match is not None:
            for line in f:
                (key, val) = line.split()
                if key in readme_param_match:
                    params[readme_param_match[key]] = val
        else:
            for line in f:
                (key, val) = line.split()
                params[key] = val

    return params


def make_rheol_string_fmt(rheoldict, law='coulomb'):
    """Make string from rheological parameters."""
    text = ''
    for name in ['delta1', 'delta2', 'delta3', 'delta4']:
        if name in rheoldict:
            new_txt = name + '_{:05.2f}_'
            text += new_txt
    if 'ksi' in rheoldict:
        new_txt = name + 'ksi_{:06.1f}_'
        text += new_txt
    text = text[:-1]

    return text


def make_rheol_string(rheoldict, law):
    """Make strings from rheological parameters."""

    keys = [key for key in rheoldict]
    for key in keys:
        if not isinstance(rheoldict[key], list):
            rheoldict[key] = [rheoldict[key]]

    nparams = len(rheoldict[keys[0]])
    txt_fmt = make_rheol_string_fmt(rheoldict)
    texts = []

    for i in range(nparams):
        if law == 'coulomb':
            txt_fmt = 'delta1_{:05.2f}'
            text = txt_fmt.format(rheoldict['delta1'][i]).replace('.', 'p')
        if law == 'voellmy':
            txt_fmt = 'delta1_{:05.2f}_ksi_{:06.1f}'
            text = txt_fmt.format(rheoldict['delta1'][i],
                                  rheoldict['ksi'][i]).replace('.', 'p')
        if law == 'pouliquen_2002':
            txt_fmt = 'delta1_{:05.2f}_L_{:05.2f}'
            text = txt_fmt.format(rheoldict['delta1'][i],
                                  rheoldict['wlong'][i]).replace('.', 'p')
        texts.append(text)

    if nparams == 1:
        texts = texts[0]

    return texts
